@@ -1561,13 +1561,12 @@ class VariableClUpdateBinhostUnstableList(Variable):
     value = ["https://testing.calculate-linux.org/"]
     # value = ["ftp://ftp.calculate-linux.ru/testing"]
 
-class VariableClUpdateBinhostListMigration(Variable):
+class VariableClUpdateBinhostMigrationList(Variable):
     """
     Список хостов с бинарными обновлениями для стабильной миграции
     """
     type = "list"
-    #TODO placeholder
-    value = ["https://testing.calculate-linux.org/"]
+    value = ["https://mirror.calculate-linux.org/migrate/"]
 
 class VariableClUpdateBinhostBase(Variable):
     """
@@ -1858,4 +1857,19 @@ class VariableClUpdateLevel(Variable):
     Текущий уровень обновления
     Используется для выбора зеркала
     """
-    value = "0"
\ No newline at end of file
+    value = "0"
+
+class VariableClUpdateUseMigrationHost(Variable):
+    """
+    Уровень обновления на бинхосте
+    """
+    type = "bool"
+    value = "off"
+
+class VariableClCheckUpdateLevel(Variable):
+    """
+    Проверять уровень обновления 
+    при соединении с binhost
+    """
+    type = "bool"
+    value = "on"
\ No newline at end of file
